import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';

import USD from 'Icon/icons/usd.svg';
import EUR from 'Icon/icons/eur.svg';
import UAH from 'Icon/icons/uah.svg';

import './index.scss';

const CURRENCIES = { UAH, EUR, USD };

const decomposeAmount = amount => (parseFloat(amount))
  .toLocaleString('ru-RU', { maximumFractionDigits: 2, minimumFractionDigits: 2 })
  .toString()
  .split(',');

/**
 * ## Usage
 *
 * ```javascript
 * import AmountFormatNew from '@Components/AmountFormatNew';
 *
 * export default class {
 *   render() {
 *     return (
 *       <AmountFormatNew
 *         amount={10.12}
 *         label="your label if you need"
 *         prefix="prefix text"
 *         inlineLabel
 *         prefixCurrency="UAH"
 *         identicalFractionFontSize // 12.12 kinda stuff, by default would be 12. small fraction 12
 *       />
 *     );
 *   }
 * }
 * ```
 */
@bemDecorator('amount-format-new')
export default class AmountFormatNew extends React.PureComponent {
  static displayName = 'AmountFormatNew';

  static propTypes = {
    amount: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.number,
    ]),
    label: PropTypes.string,
    inlineLabel: PropTypes.bool,
    prefix: PropTypes.string,
    prefixCurrency: PropTypes.oneOf(['UAH', 'USD', 'EUR', '']),
    suffix: PropTypes.string,
    suffixCurrency: PropTypes.oneOf(['UAH', 'USD', 'EUR', '']),
    identicalFractionFontSize: PropTypes.bool,
    inherit: PropTypes.bool,
  };

  static defaultProps = {
    amount: '',
    label: '',
    inlineLabel: false,
    prefix: '',
    prefixCurrency: '',
    suffix: '',
    suffixCurrency: '',
    identicalFractionFontSize: false,
    inherit: false,
  };

  render(bem) {
    const {
      label, amount, inlineLabel,
      prefix, prefixCurrency, suffix, suffixCurrency,
      identicalFractionFontSize, inherit
    } = this.props;

    const [wholePart, fraction] = decomposeAmount(amount);

    const style = inherit
      ? {
        color: 'inherit',
        fontWeight: 'inherit',
        fontSize: 'inherit',
      }
      : undefined;

    return (
      <div style={style} className={bem({ 'inline-label': inlineLabel })}>
        {label && <p style={style} className={bem('label', { inline: inlineLabel })}>{label}</p>}

        <div style={style} className={bem('value-wrap')}>
          <span style={style} className={bem('value')}>
            {prefix && <span style={style} className={bem('prefix')}>{prefix}{' '}</span>}

            {prefixCurrency && (
              <span className={bem('currency')}>
                <img
                  src={CURRENCIES[prefixCurrency]}
                  alt={prefixCurrency}
                  className={bem('currency-img', { 'full-height': identicalFractionFontSize })}
                />
              </span>
            )}

            {parseFloat(amount) >= 0 ? (
              <span style={style} className={bem('amount')}>
                {prefixCurrency && ' '}
                <span className={bem('amount-whole-part')}>{wholePart}</span>
                {'.'}
                <span
                  style={style}
                  className={bem('amount-fraction', { 'identical-font-size': identicalFractionFontSize })}
                >{fraction}</span>
                {suffixCurrency && ' '}
              </span>
            ) : (
              <span
                style={style}
                className={bem('amount-empty')}
              >{prefixCurrency && ' '}— — . — —{suffixCurrency && ' '}</span>
            )}

            {suffixCurrency && (
              <span className={bem('currency')}>
                <img
                  src={CURRENCIES[suffixCurrency]}
                  alt={suffixCurrency}
                  className={bem('currency-img', { 'full-height': identicalFractionFontSize })}
                />
              </span>
            )}

            {suffix && <span style={style} className={bem('suffix')}>{' '}{suffix}</span>}
          </span>
        </div>
      </div>
    );
  }
}

