import React from 'react';
import PropTypes from 'prop-types';
import Paper from 'material-ui/Paper';
import { MenuItem } from 'material-ui/Menu';
import Downshift from 'downshift';
import TextInput from '../TextInput';


/**
 * ## Usage
 *
 * ```javascript
 * import { Field, formValues } from 'redux-form/immutable';
 * import AutoComplete from '@Components/AutoComplete';
 *
 * @formValues('account')
 * export default class {
 *   reloadAccounts(name) {
 *      const apiEndpoint = new URL('http://api.com/countries');
 *      apiEndpoint.searchParams.append('query', name);
 *      fetch(apiEndpoint.toString()).then(response => {
 *        this.setState({
 *          list: this.state.list.merge(response),
 *          name,
 *        });
 *      });
 *   }
 *
 *   getList() {
 *      const { account } = this.props;
 *      const { name } = this.props;
 *      return this.state.list.filter(
 *        option => option.label.like(name) || option.value === account
 *      );
 *   }
 *
 *
 *   render() {
 *     const { list } = this.state;
 *     return (
 *       <Field
 *          name="account"
 *          component={AutoComplete}
 *          label="Select account for payment"
 *          onQueryChange={() => this.reloadAccounts()}
 *          customNormalize={value = value.replace(/[a-zA-Z]/g, '')}
 *       />
 *     );
 *   }
 * }
 * ```
 *
 * ## Other Props
 * All passed props that are not specified in PropTypes will be spreaded to TextInput component
 */
export default class AutoComplete extends React.PureComponent {
  static displayName = 'AutoComplete';

  static propTypes = {
    /**
     * List of suggestion options, if field is initialized with predefined value,
     * options with
     */
    list: PropTypes.arrayOf(PropTypes.shape({
      value: PropTypes.string,
      label: PropTypes.string,
    })),
    /** Provided by redux-form */
    input: PropTypes.object,
    /** Provided by redux-form */
    meta: PropTypes.object,
    /**
     * Callback function fired when users changes search query
     *
     * Signature:
     * ```
     * function(string: inputValue) => void
     * ```
     */
    onQueryChange: PropTypes.func,
    limit: PropTypes.number,
    customNormalize: PropTypes.func
  };

  static defaultProps = {
    list: [],
    input: {},
    meta: {},
    onQueryChange: () => {},
    limit: 5,
    customNormalize: value => value
  };

  state = {
    menuIsOpen: false,
  };

  getSuggestions(inputValue, list) { // eslint-disable-line
    const { limit } = this.props;
    return list.slice(0, limit);
  }

  findValue(value) {
    return (this.props.list.find(item => item.label === value) || {}).value;
  }

  renderInput(inputProps) { // eslint-disable-line
    const { InputProps, ref, ...other } = inputProps;
    const { customNormalize, resetInput } = this.props;

    return (
      <TextInput
        {...other}
        inputRef={ref}
        input={{
          ...InputProps,
          value: resetInput ? '' : customNormalize(InputProps.value),
          onBlur: ({ target }) => {
            this.setState({ menuIsOpen: false });
            InputProps.onBlur(this.findValue(target.value));
          },
          onClick: ({ target }) => {
            if (!target.disabled) {
              this.setState({ menuIsOpen: true });
              InputProps.onFocus(this.findValue(target.value));
            }
          },
        }}
      />
    );
  }

  renderSuggestion(params) { // eslint-disable-line
    const {
      suggestion, index, itemProps, highlightedIndex, selectedItem
    } = params;
    const isHighlighted = highlightedIndex === index;
    const isSelected = selectedItem === suggestion.label;

    return (
      <MenuItem
        {...itemProps}
        key={suggestion.label}
        selected={isHighlighted}
        component="div"
        style={{
          fontWeight: isSelected ? 500 : 400,
        }}
      >
        {suggestion.label}
      </MenuItem>
    );
  }

  inputValueChange(inputValue) {
    const { list, input } = this.props;
    const matchingItem = list.find(item => item.label === inputValue);
    if (matchingItem && input.value !== matchingItem.value) {
      input.onChange(matchingItem.value);
    } else if (!matchingItem && inputValue && inputValue.length > 0) {
      this.props.onQueryChange(inputValue);
    } else {
      this.props.onQueryChange('');
    }

    this.setState({ menuIsOpen: false });
  }

  render() {
    const {
      list, meta, input: { value, onChange, ...otherFieldInputProps },
      ...otherProps
    } = this.props;
    const { menuIsOpen } = this.state;
    const selectedOption = list.find(item => item.value === value);

    return (
      <Downshift
        defaultInputValue={selectedOption ? selectedOption.label : value}
        onInputValueChange={inputValue => this.inputValueChange(inputValue)}
      >
        {({
            getInputProps, getItemProps, isOpen, inputValue, selectedItem, highlightedIndex
          }) => (
            <div>
              {this.renderInput({
                fullWidth: true,
                InputProps: getInputProps({ ...otherFieldInputProps }),
                meta,
                ...otherProps,
              })}
              {(isOpen || menuIsOpen) ? (
                <Paper square>
                  {this.getSuggestions(inputValue, list).map((suggestion, index) =>
                    this.renderSuggestion({
                      suggestion,
                      index,
                      itemProps: getItemProps({ item: suggestion.label }),
                      highlightedIndex,
                      selectedItem,
                    }))}
                </Paper>
              ) : null}
            </div>
        )}
      </Downshift>
    );
  }
}
