import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import { CircularProgress } from 'material-ui/Progress';

import UIButton from 'material-ui/Button';
import { Link } from 'react-router-dom';

import Icon from '../Icon';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import Button from '@Components/Button';
 *
 * export default class {
 *   render() {
 *     return (
 *       <div>
 *           <Button onClick={() => {}}>Example button</Button>
 *           <Button to="/account" theme="link">Example link</Button>
 *       </div>
 *     );
 *   }
 * }
 * ```
 */
export default class Button extends React.PureComponent {
  static displayName = 'Button';

  static propTypes = {
    /** Button content */
    children: PropTypes.any.isRequired,
    /** Enable button disabled state, usefull when form is submitting */
    disabled: PropTypes.bool,
    onClick: PropTypes.func,
    to: PropTypes.string,
    href: PropTypes.string,
    target: PropTypes.string,
    block: PropTypes.bool,
    theme: PropTypes.oneOf(['default', 'link', 'default-link', 'border', 'square', 'round', 'ellipse', 'flat-button', 'button-link', 'link-arrow', 'round-transparent']),
    size: PropTypes.oneOf(['small', 'middle']),
    arrowSize: PropTypes.number,
    type: PropTypes.oneOf(['button', 'submit']),
    color: PropTypes.oneOf(['default', 'white', 'green', 'red', 'transparent']),
    loading: PropTypes.bool,
    id: PropTypes.string,
    isNew: PropTypes.bool,
    opacity: PropTypes.bool,
  };

  static defaultProps = {
    theme: 'default',
    size: 'middle',
    arrowSize: 14,
    type: 'button',
    color: 'default',
    target: '_blank',
    disabled: false,
    block: false,
    onClick: () => {
    },
    to: undefined,
    href: undefined,
    loading: false,
    id: undefined,
    isNew: false,
    opacity: false,
  };

  get rootComponent() {
    const { to, href } = this.props;

    if (href) return 'a';
    if (to) return Link;

    return 'button';
  }

  render() {
    const {
      children, disabled, onClick, to,
      theme, size, block, type, color, id,
      loading, arrowSize, isNew, target, href, opacity,
    } = this.props;

    const ROUND_BUTTONS_THEME = ['round', 'round-transparent'];

    return (
      <UIButton
        id={id}
        variant={ROUND_BUTTONS_THEME.includes(theme) ? 'fab' : 'flat'}
        disabled={disabled || loading}
        classes={{
          root: classNames('button', `button_theme_${theme}`, `button_color_${color}`, `button_size_${size}`, block && 'button_block', opacity && 'button_with_opacity'),
          label: 'button__label',
          disabled: 'button_disabled',
        }}
        onClick={onClick}
        to={to}
        target={href ? target : undefined}
        href={href || undefined}
        type={type}
        component={this.rootComponent}
        disableRipple={['link', 'default-link', 'link-arrow'].includes(theme)}
      >
        {ROUND_BUTTONS_THEME.includes(theme) && loading && (
          <CircularProgress
            classes={{ root: 'button__loading' }}
            size={50}
            thickness={2}
          />
        )}

        {children}

        {theme === 'link-arrow' && (
          <span className="button__icon-arrow">
            <Icon name="arrow-down" size={arrowSize} />
          </span>
        )}

        {isNew && (
          <span className="button__icon-new">new</span>
        )}
      </UIButton>
    );
  }
}
