import React, { Component } from 'react';
import PropTypes from 'prop-types';
import isEmpty from 'is-empty';
import DateRangePicker from 'react-bootstrap-daterangepicker';
import moment from 'moment';

import './index.scss';

export default class CustomDatePicker extends Component {
  static propTypes = {
    children: PropTypes.node,
  };

  static defaultProps = {
    children: null,
  };

  state = {
    startDate: moment().startOf('isoweek'),
    endDate: moment(),
    dateSelected: false,
    datePicker: true,
    labelDate: '',
  };

  componentWillMount() {
    const { range } = this.props;
    const format = 'DD-MM-YYYY';

    if (!isEmpty(range)) {
      this.setState({
        dateSelected: true,
        startDate: moment(range.startDate, format),
        endDate: moment(range.endDate, format),
      });
    }
  }

  componentWillReceiveProps(newProps) {
    if (isEmpty(newProps.range)) {
      this.setState({ dateSelected: false });
    }

    this.restartDatePicker();
  }

  componentWillUpdate(nextProps, nextState) {
    if (this.state.startDate !== nextState.startDate || this.state.endDate !== nextState.endDate) {
      this.notifyParent(nextState);
    }
  }

  get ranges() {
    const { filter } = this.props;
    const result = {};
    const rangesList = {
      today: [moment(), moment()],
      yesterday: [moment().subtract(1, 'days'), moment().subtract(1, 'days')],
      thisWeek: [moment().startOf('isoweek'), moment()],
      prevWeek: [moment().startOf('isoweek').day(-6), moment().endOf('isoweek').day(-7)],
      month: [moment().startOf('month'), moment()],
      prevMonth: [moment().subtract(1, 'month').startOf('month'), moment().subtract(1, 'month').endOf('month')],
      quarter: [moment().startOf('quarter'), moment()],
      prevQuarter: [moment().startOf('quarter').quarter(moment().quarter() - 1), moment().endOf('quarter').quarter(moment().quarter() - 1)],
      year: [moment([moment().year(), 0, 1]), moment()],
    };

    for (let key in rangesList) {
      if (!Array.isArray(filter) || filter.includes(key)) {
        result[key] = rangesList[key];
      }
    }

    return result;
  }

  get locale() {
    const { t } = this.props;
    return {
      customRangeLabel: 'choose-range',
      daysOfWeek: this.daysTitle,
      monthNames: this.monthsTitle,
      firstDay: 1,
      applyLabel: 'ok',
      cancelLabel: 'cancel',
    };
  }

  get monthsTitle() {
    return this.getYearItems('month', [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12]);
  }

  get daysTitle() {
    return this.getYearItems('day', [1, 2, 3, 4, 5, 6, 7]);
  }

  getLabel(start, end) {
    const startDate = start || this.state.startDate;
    const endDate = end || this.state.endDate;
    const { dateSelected } = this.state;
    const { singleDatePicker, customLabel } = this.props;
    const format = 'DD.MM.YYYY';

    let label = '';

    if (!dateSelected) {
      label = customLabel;
    } else if (singleDatePicker) {
      label = startDate.format(format);
    } else if (startDate.format('MM') === endDate.format('MM')) {
      label = startDate.format('MMMM YYYY');
    } else {
      label = `${startDate.format(format)} - ${endDate.format(format)}`;
    }

    return label;
  }

  getYearItems = (type, collection) => (
    collection.map(i => `${type}_${i}`)
  );

  async restartDatePicker() {
    await this.setState({ datePicker: false });
    await this.setState({ datePicker: true });
  }

  notifyParent = obj => {
    this.props.handleApply(obj);
  };

  handleApply = (event, picker, dateSelected = true) => {
    const { startDate, endDate } = picker;

    this.setState({
      startDate,
      endDate,
      dateSelected,
      labelDate: this.getLabel(startDate, endDate),
    });
  };

  render() {
    const { startDate, endDate, dateSelected, datePicker } = this.state;
    const { children, singleDatePicker, customLabel } = this.props;
    const format = 'DD.MM.YYYY';
    let label = '';

    if (customLabel) {
      label = this.getLabel();
    } else {
      label = dateSelected ? `${startDate.format(format)} - ${endDate.format(format)}` : 'choose-range';
    }

    return (
      <div className="date-picker-wrapper">
        <DateRangePicker
          onApply={this.handleApply}
          locale={this.locale}
          ranges={this.ranges}
          maxDate={moment()}
          startDate={startDate}
          endDate={endDate}
          dateSelected={dateSelected}
          opens="left"
          parentEl=".date-picker-wrapper"
          singleDatePicker={singleDatePicker}
        >
              <span className="date-title-range">
                {children || label}
              </span>
        </DateRangePicker>
      </div>
    );
  }
}

