import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';

import Input  from 'material-ui/Input';
import { MenuItem } from 'material-ui/Menu';
import { FormControl, FormHelperText } from 'material-ui/Form';
import SelectUI from 'material-ui/Select';
import { LinearProgress } from 'material-ui/Progress';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import Example from '@Components/CustomSelect';
 *
 * export default class {
 *   render() {
 *     return (
 *       <CustomSelect />
 *     );
 *   }
 * }
 * ```
 */
@bemDecorator('custom-select')
export default class CustomSelect extends React.PureComponent {
  static displayName = 'CustomSelect';

  static propTypes = {
    disabled: PropTypes.bool,
    input: PropTypes.object,
    innerProps: PropTypes.object,
    meta: PropTypes.object,
    id: PropTypes.string,
    list: PropTypes.array,
    customLabel: PropTypes.string,
    theme: PropTypes.oneOf([
      'default',
      'payments-form',
      'autofill-form',
    ]),
    loading: PropTypes.bool,
  };

  static defaultProps = {
    id: undefined,
    disabled: false,
    input: {
      value: '',
      name: '',
    },
    meta: {},
    innerProps: {},
    list: [],
    customLabel: '',
    theme: 'default',
    loading: false,
  };

  state = {
    editableMode: !this.props.list.includes(this.props.input.value),
  };

  render(bem) {
    const {
      input, innerProps, meta, list,
      id, disabled, loading, customLabel, change,
    } = this.props;
    const { editableMode } = this.state;

    const error = (
      meta.touched || (meta.visited && !meta.active) || meta.submitFailed
    ) && meta.error;

    return (
      <FormControl
        disabled={disabled || loading}
        error={error}
        className={bem({ theme: this.props.theme, loading, 'editable-mode': editableMode })}
      >
        {
          customLabel && (
            <section className={bem('editable-input')}>
              <Input
                error={error || undefined}
                disabled={disabled}
                classes={{
                  root: bem('input'),
                  focused: bem('input', { focused: true }),
                  error: bem('input', { error: true }),
                }}
                inputProps={{
                  ...input,
                  name: input.name,
                }}
                {...innerProps}
                {...input}
              />
            </section>
          )
        }
        <SelectUI
          {...innerProps}
          {...input}
          input={<Input
            classes={{
              root: bem('input'),
              focused: bem('input', { focused: true }),
              error: bem('input', { error: true }),
            }}
            name={input.name}
            id={id}
          />}
          onChange={event => {
            const { value } = event.target;

            if (value === customLabel && editableMode) {
              return;
            }

            this.setState({ editableMode: value === customLabel });

            input.onChange(event);

            if (value === customLabel) {
              change(input.name, '');
            }
          }}
        >
          {
            list.map(item => (
              <MenuItem key={item} value={item}>{item}</MenuItem>
            ))
          }

          {
            customLabel && (
              <MenuItem
                className={bem('editable')}
                key={customLabel}
                value={customLabel}
              >
                {customLabel}
              </MenuItem>
            )
          }
        </SelectUI>

        {error && <FormHelperText className={bem('help-text', { error: true })}>{error}</FormHelperText>}

        {loading && <LinearProgress classes={{ root: bem('progress') }} variant="query" />}
      </FormControl>
    );
  }
}
