import React from 'react';
import PropTypes from 'prop-types';
import { DatePicker } from 'material-ui-pickers';
import { InputLabel } from 'material-ui/Input';
import classNames from 'classnames';
import './index.scss';

const ARROW_BTN_SIZE = 24;
const ARROW_BTN_WIDTH = 8;
const ARROW_BTN_COLOR = '#000';
const ARROW_BTN_STYLE = {
  boxSizing: 'border-box',
  padding: 5,
};
const LEFT_ARROW_POINTS = '45.63,75.8 0.375,38.087 45.63,0.375';
const RIGHT_ARROW_POINTS = '0.375,0.375 45.63,38.087 0.375,75.8';
const ArrowButton = ({ points }) => (
  <svg
    width={ARROW_BTN_SIZE}
    height={ARROW_BTN_SIZE}
    style={ARROW_BTN_STYLE}
    viewBox="0 0 50 80"
    xmlSpace="preserve"
  >
    <polyline
      fill="none"
      stroke={ARROW_BTN_COLOR}
      strokeWidth={ARROW_BTN_WIDTH}
      strokeLinecap="round"
      strokeLinejoin="round"
      points={points}
    />
  </svg>
);
/**
 * ## Usage
 *
 * ```javascript
 * import Button from '@Components/DatePicker';
 *
 * link on source library https://github.com/dmtrKovalenko/material-ui-pickers
 *
 * export default class {
 *   render() {
 *     return (
 *       <div>
 *           <DatePicker input={{
 *            onChange(date){}
 *           }}>
 *              Example button
 *           </Button>
 *       </div>
 *     );
 *   }
 * }
 * ```
 */
export default class CustomDatePicker extends React.PureComponent {
  static displayName = 'DatePicker';

  static propTypes = {
    okLabel: PropTypes.string,
    cancelLabel: PropTypes.string,
    clearLabel: PropTypes.string,
    label: PropTypes.string,
    openToYearSelection: PropTypes.bool,
    autoOk: PropTypes.bool,
    disablePast: PropTypes.bool,
    disableFuture: PropTypes.bool,
    disabled: PropTypes.bool,
    fullwidth: PropTypes.bool,
    minDate: PropTypes.string,
    maxDate: PropTypes.string,
    format: PropTypes.oneOf([
      'MMMM Do', 'MMMM DD', 'MMMM.DD', 'MM Do YY', 'MM.Do.YY', 'MM.DD.YY', 'MM.DD.YYYY',
    ]),
    input: PropTypes.shape({
      onChange: PropTypes.func.isRequired,
      value: PropTypes.string,
    }).isRequired,
    meta: PropTypes.shape({
      touched: PropTypes.bool,
      visited: PropTypes.bool,
      active: PropTypes.bool,
      error: PropTypes.string,
    }),
  };

  static defaultProps = {
    label: '',
    okLabel: undefined,
    cancelLabel: undefined,
    clearLabel: undefined,
    format: 'MM.DD.YY',
    autoOk: true,
    disablePast: false,
    disabled: false,
    disableFuture: false,
    openToYearSelection: false,
    fullwidth: false,
    minDate: '1950-01-01',
    maxDate: '2050-01-01',
    input: {
      value: null,
      onChange: () => {
      },
    },
    meta: {
      touched: false,
      visited: false,
      active: false,
      error: '',
    },
  };

  handleDateChange = (date) => {
    this.props.input.onChange(date);
  };

  render() {
    const {
      input, innerProps, format, autoOk, label, okLabel, cancelLabel, clearLabel,
      disabled, meta, fullwidth, openToYearSelection, disablePast, disableFuture,
      minDate, maxDate,
    } = this.props;
    const { value } = input;
    const error = (
      meta.touched || (meta.visited && !meta.active)
    ) && meta.error;

    return (
      <div className="tas-date-picker">
        <InputLabel
          className={classNames('tas-date-picker__label', { 'tas-date-picker__label_focused': value })}
          error={error}
        >{label}
        </InputLabel>
        <DatePicker
          className={classNames('tas-date-picker__DP', {
            'tas-date-picker__DP_fullwidth': fullwidth,
          })}
          disabled={disabled}
          error={error}
          okLabel={okLabel}
          cancelLabel={cancelLabel}
          clearLabel={clearLabel}
          format={format}
          autoOk={autoOk}
          openToYearSelection={openToYearSelection}
          disablePast={disablePast}
          disableFuture={disableFuture}
          minDate={minDate}
          maxDate={maxDate}
          animateYearScrolling
          clearable
          leftArrowIcon={(<ArrowButton points={LEFT_ARROW_POINTS} />)}
          rightArrowIcon={(<ArrowButton points={RIGHT_ARROW_POINTS} />)}
          {...input}
          {...innerProps}
          onChange={this.handleDateChange}
        />
      </div>
    );
  }
}
