import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { IconButton } from 'material-ui';
import classNames from 'classnames';
import { DatePicker } from 'material-ui-pickers';
import { InputLabel } from 'material-ui/Input';
import { FormControl, FormHelperText } from 'material-ui/Form';
import moment from 'moment';
import Icon from '../Icon';

import './index.scss';

class DateRangePicker extends Component {
  static displayName = 'DateRangePicker';

  static propTypes = {
    theme: PropTypes.oneOf(['default', 'label']),
    classes: PropTypes.object,
    okLabel: PropTypes.string,
    cancelLabel: PropTypes.string,
    clearLabel: PropTypes.string,
    label: PropTypes.string,
    disabled: PropTypes.bool,
    fullwidth: PropTypes.bool,
    input: PropTypes.shape({
      onChange: PropTypes.func.isRequired,
      value: PropTypes.object,
    }).isRequired,
    meta: PropTypes.shape({
      touched: PropTypes.bool,
      visited: PropTypes.bool,
      active: PropTypes.bool,
      error: PropTypes.string,
    }),
  };

  static defaultProps = {
    theme: 'default',
    classes: {},
    label: '',
    okLabel: undefined,
    cancelLabel: undefined,
    clearLabel: undefined,
    disabled: false,
    fullwidth: false,
    input: { // eslint-disable-line
      value: null,
      onChange: () => {
      },
    },
    meta: {
      touched: false,
      visited: false,
      active: false,
      error: '',
    },
  };

  state = {
    startDate: null,
    endDate: null,
  };

  get value() {
    if (this.props.input && this.props.input.value) {
      return this.props.input.value;
    }
    return {
      start: null,
      end: null
    };
  }

  handleChange = () => {
    this.props.input.onChange({ start: this.state.startDate.format('x'), end: this.state.endDate.format('x') });
  };

  handleDateChange = (currentDate) => {
    const { startDate, endDate } = this.state;

    if (!startDate) {
      this.setState({ startDate: currentDate });
    } else if (startDate.isSame(currentDate)) {
      this.setState({ startDate: endDate || null, endDate: null });
    } else if (startDate.isAfter(currentDate)) {
      this.setState({ startDate: currentDate, endDate: startDate });
    } else if (endDate && endDate.isSame(currentDate)) {
      this.setState({ endDate: null });
    } else {
      this.setState({ endDate: currentDate });
    }
    // this.setState({ selectedDate: currentDate });
  };

  formatWeekSelectLabel = () => {
    const { start, end } = this.value;
    if (!start || !end) {
      return '';
    }
    const startDate = moment(Number(start));
    const endDate = moment(Number(end));

    if (startDate && endDate && startDate.isValid() && endDate.isValid()) {
      return `${startDate.format('DD.MM.YY')} - ${endDate.format('DD.MM.YY')}`;
    }
    return '';
  };

  handleClose = () => {
    this.setState({
      startDate: null,
      endDate: null
    });
  };

  renderWrappedDefaultDay = (date, selectedDate, dayInCurrentMonth) => {
    const { startDate, endDate } = this.state;

    const dayIsBetween = (
      date.isSame(startDate) ||
      date.isSame(endDate) ||
      (date.isAfter(startDate) && date.isBefore(endDate))
    );

    const firstDay = date.isSame(startDate, 'day');
    const lastDay = date.isSame(endDate, 'day');

    const wrapperClassName = classNames({
      'dp-day__hl': dayIsBetween,
      'dp-day__hl_f': firstDay,
      'dp-day__hl_l': lastDay,
    });

    const dayClassName = classNames('dp-day', {
      'dp-day__other-month ': !dayInCurrentMonth,
      'dp-day__hl-other-month': !dayInCurrentMonth && dayIsBetween,
    });

    return (
      <div className={wrapperClassName} >
        <IconButton className={dayClassName} onClick={() => this.handleDateChange(date)} >
          <span > {date.format('DD')} </span >
        </IconButton >
      </div >
    );
  };

  render() {
    const {
      label, input,
      innerProps, meta, helperText, theme
    } = this.props;
    const { value } = input;
    const error = (
      meta.touched || (meta.visited && !meta.active)
    ) && meta.error;

    const BLOCK_NAME = 'tas-date-picker';
    return (
      <FormControl className={`tas-date-picker tas-date-picker_theme_${theme}`} >
        <InputLabel
          className={classNames('tas-date-picker__label', { 'tas-date-picker__label_focused': value })}
          error={error}
        >{label}
        </InputLabel >
        <DatePicker
          {...input}
          {...innerProps}
          {...meta}
          onChange={this.handleChange}
          renderDay={this.renderWrappedDefaultDay}
          labelFunc={this.formatWeekSelectLabel}
          onClose={this.handleClose}
          leftArrowIcon={(<Icon name="arrow-down" rotate="90deg" />)}
          rightArrowIcon={(<Icon name="arrow-down" rotate="-90deg" />)}
          animateYearScrolling
          inputProps={{
            classes: {
              root: `${BLOCK_NAME}__input`,
              focused: `${BLOCK_NAME}__input_focused`,
              error: `${BLOCK_NAME}__input_error`,
            }
          }}
          className={`${BLOCK_NAME}__input`}
        />

        {helperText && <FormHelperText className={`${BLOCK_NAME}__help-text`} >{helperText}</FormHelperText >}
        {error &&
        <FormHelperText className={`${BLOCK_NAME}__help-text ${BLOCK_NAME}__help-text_error`} >
          {error}
        </FormHelperText >
        }
      </FormControl >
    );
  }
}

export { DateRangePicker };

export default DateRangePicker;
