import React from 'react';
import PropTypes from 'prop-types';

import Input from 'material-ui/Input';
import { MenuItem } from 'material-ui/Menu';
import SelectUI from 'material-ui/Select';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import Example from '@Components/Select';
 *
 * export default class {
     * state = {
        value: 1,
      };

      handleChange(value) {
        this.setState({ value });
      };

 *   render() {
 *     return (
 *       <DropDown
          value={this.state.value}
          onChange={value => { handleChange(value); }}
          list={[
            { label: 'One', value: 1 },
            { label: 'Two', value: 2 },
            { label: 'Three', value: 3 },
          ]}
          theme="your-custom-theme"
        />
 *     );
 *   }
 * }
 * ```
 */
export default class DropDown extends React.PureComponent {
  static displayName = 'DropDown';

  static propTypes = {
    disabled: PropTypes.bool,
    withIcon: PropTypes.bool,
    value: PropTypes.any,
    id: PropTypes.string,
    theme: PropTypes.string,
    list: PropTypes.arrayOf(PropTypes.shape({
      value: PropTypes.any,
      label: PropTypes.string,
    })),
    emptyText: PropTypes.any,
    onChange: PropTypes.func,
  };

  static defaultProps = {
    id: undefined,
    value: null,
    theme: 'default',
    emptyText: undefined,
    disabled: false,
    withIcon: false,
    onChange: () => {},
    list: [],
  };

  render() {
    const {
      list, id, emptyText, disabled, value, withIcon, theme, onChange,
    } = this.props;
    return (
      <SelectUI
        value={value}
        disabled={disabled}
        onChange={(e) => { onChange(e.target.value); }}
        className={`drop-down drop-down_theme_${theme}`}
        classes={{
          icon: `drop-down__icon ${withIcon ? 'drop-down__icon_show' : ''}`,
        }}
        MenuProps={{
          classes: { paper: `dropDownModal-menu dropDownModal-menu_theme_${theme}` },
          MenuListProps: { className: 'dropDownModal-menu__list-root' },
        }}
        input={<Input
          classes={{
            input: `drop-down__input ${withIcon ? 'drop-down__input_showIcon' : ''}`,
            focused: 'drop-down_focused',
            disabled: 'drop-down_disabled',
          }}
          id={id}
        />}
      >
        {emptyText && <MenuItem className="dropDownModal-menu__list-root__emptyText" value=""><em>{emptyText}</em></MenuItem>}
        {list.map(({ value, label }) => (
          <MenuItem
            key={value}
            value={value}
            classes={{ selected: 'dropDownModal-menu__list-root_selected' }}
          >
            {label}
          </MenuItem>
        ))}
      </SelectUI>
    );
  }
}
