import React from 'react';
import PropTypes from 'prop-types';
import GoogleMapLoader from 'react-google-maps-loader';
import GooglePlacesSuggest from 'react-google-places-suggest';

const DEBOUNCE_TIMEOUT = 250;

/**
 * Wrapper for react-google-maps-loader and react-google-places-suggest libraries
 *
 * ## Usage
 * ```javascript
 * import GooglePlaceSuggestion, { getAddress } from '@Components/GooglePlaceSuggestion'
 *
 * @formValues('search')
 * @connect(, dispatch => ({
 *  doSelectSuggestion(suggestion) {
 *    dispatch(change('FORM_NAME', 'search', suggestion));
 *  }
 * }))
 * class SuggestionInput extends React.PureComponent {
 *  render() {
 *    const { search, doSelectSuggestion } = this.props;
 *    return (
 *      <GooglePlaceSuggestion
 *        autocompletionRequest={{
 *          componentRestrictions: { country: 'ua' }, // Restrictions for suggestions
 *          types: ['(cities)'], // Suggestions types
 *        }}
 *        search={search}
 *        onSelectSuggest={doSelectSuggestion}
 *        suggestionFormatter={getAddress}
 *      >
 *        <Field
 *          name="search"
 *          component={TextInput}
 *        />
 *      </GooglePlaceSuggestion>
 *    );
 *  }
 * }
 * ```
 *
 */
export default class GooglePlaceSuggestion extends React.Component {
  static displayName = 'GooglePlaceSuggestion';

  static propTypes = {
    /**
     * Input element
     */
    children: PropTypes.node.isRequired,
    /**
     * Triggers when users selects one of suggestion, argument is google place object
     * processed with suggestionFormatter function
     */
    onSelectSuggest: PropTypes.func.isRequired,
    /**
     * Google Maps Api autocompletion request,
     * {
     *  componentRestrictions: { country: 'ua' }, // Restrictions for suggestions
     *  types: ['(cities)'], // Suggestions types
     *  input: 'Qwerty' // Search query string
     * }
     * Left property input empty
     */
    autocompletionRequest: PropTypes.object.isRequired,
    /** No places found text */
    textNoResults: PropTypes.string,
    search: PropTypes.string.isRequired,
    /** Transforms google place objects to needed format */
    suggestionFormatter: PropTypes.func,
    /** Google Maps API key */
    googleMapsKey: PropTypes.string,
  };

  static defaultProps = {
    suggestionFormatter: getFormattedAddress,
    textNoResults: null,
    /** default Tascombank Google Maps API key */
    googleMapsKey: 'AIzaSyCEMwaR7ipFTdhML9ftKk8Y7dax6HaJYj4',
  };

  state = {
    search: '',
    selectedSuggestion: null,
  };

  componentWillReceiveProps(nextProps) {
    if (nextProps.search === this.state.selectedSuggestion) {
      return;
    }
    if (nextProps.search !== this.state.search) {
      clearTimeout(this.timeout);
      this.timeout = setTimeout(() => {
        this.setState({ search: nextProps.search });
      }, DEBOUNCE_TIMEOUT);
    }
  }

  shouldComponentUpdate() {
    setTimeout(() => {
      this.forceUpdate();
    }, 500);

    return false;
  }

  onSelect(suggest) {
    const formattedSuggestion = this.props.suggestionFormatter(suggest);
    // Blocks state search update until the search will not be changed
    this.setState({ search: '', selectedSuggestion: formattedSuggestion });
    this.props.onSelectSuggest(formattedSuggestion);
  }

  timeout = null;

  render() {
    const {
      children, autocompletionRequest, textNoResults, onSelectSuggest, search, googleMapsKey,
      ...otherProps
    } = this.props;
    return (
      <div
        onKeyDown={(e) => {
          if (e.keyCode === 13) {
            e.preventDefault();
          }
        }}
      >
        <GoogleMapLoader
          params={{
            key: googleMapsKey,
            libraries: 'places,geocode',
            language: 'uk-UA',
          }}
          render={googleMaps => googleMaps && (
            <GooglePlacesSuggest
              autocompletionRequest={{
                ...autocompletionRequest,
                input: this.state.search,
              }}
              googleMaps={googleMaps}
              onSelectSuggest={suggest => this.onSelect(suggest)}
              textNoResults={textNoResults}
              {...otherProps}
            >
              {children}
            </GooglePlacesSuggest>
          )}
        />
      </div>
    );
  }
}

export const cityFormatter = suggestion => suggestion.formatted_address.replace(/,?\s[\d]{5}$/, '');

export function getAddress(suggestion) {
  const addressComponents = ['route'];
  return suggestion.address_components && suggestion.address_components
    .filter(component => addressComponents.includes(component.types[0]))
    .map(component => component.long_name)
    .reverse()
    .join();
}

export function getFormattedAddress(suggestion) {
  return suggestion.formatted_address;
}
