import React from 'react';
import PropTypes from 'prop-types';
import Mask from 'react-nebo15-mask';

import TextInput from '../TextInput';

/**
 * ## Mask
 * For number use `1`, for string - `w`
 * Example:
 *
 * Phone number mask:
 * ```
 * mask="+38 (011) 111 11 11" // +38 (093) 999 99 99
 * ```
 *
 * Date mask
 * ```
 * mask="11-11-1111"  // 12.12.2012
 * ```
 *
 * Car number
 * ```
 * mask="ww 11 11 ww" // AB 32 23 BA
 * ```
 *
 * ## Usage
 *
 * ```javascript
 * import { Field } from 'redux-form/immutable';
 * import MaskedInput from '@Components/MaskedInput';
 *
 * export default class {
 *   render() {
 *     return (
 *       <Field
 *          name="cardNumber"
 *          component={MaskedInput}
 *          label="Card number"
 *          mask="1111 1111 1111 1111"
 *       />
 *     );
 *   }
 * }
 * ```
 */
export default class MaskedInput extends React.Component {
  static displayName = 'MaskedInput';

  static propTypes = {
    ...TextInput.propTypes,
    label: PropTypes.string,
    disabled: PropTypes.bool,
    prefix: PropTypes.string,
    suffix: PropTypes.string,
    input: PropTypes.object,
    autoFocus: PropTypes.bool,
    theme: PropTypes.oneOf([
      'default',
      'payments-form',
      'payments-form-amount',
    ]),

    /**
     * For number mask is `1`
     * For string - `w`
     */
    mask: PropTypes.string.isRequired,
    withPlaceholder: PropTypes.string,
    /**
     * Show mask on focus input
     */
    showOnFocus: PropTypes.bool,
    /**
     * Hide mask on blur input
     */
    hideOnBlur: PropTypes.bool,
    /**
     * Always show mask
     */
    showAlways: PropTypes.bool,
  };

  static defaultProps = {
    ...TextInput.defaultProps,
    withPlaceholder: undefined,
    showOnFocus: true,
    hideOnBlur: true,
    showAlways: false,
  };

  render() {
    const {
      mask, withPlaceholder, showOnFocus,
      hideOnBlur, showAlways,
      ...otherProps
    } = this.props;

    return (
      <TextInput
        {...otherProps}
        input={{
          ...otherProps.input,
          onBlur: e => e.stopPropagation(),
          onChange: e => e.stopPropagation(),
        }}
        inputComponent={Mask}
        randomName
        inputProps={{
          mask,
          withPlaceholder,
          hideOnBlur,
          showAlways,
          showOnFocus,
          onChange: (value) => {
            if (value !== this.props.input.value) {
              this.props.input.onChange(value);
            }
          },
          onBlur: (value) => this.props.input.onBlur(value),
        }}
      />
    );
  }
}
