import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';

import Modal from './index';
import Button from '../Button';
import Title from '../Title';

/**
 * ## Usage
 *
 * ```javascript
 * import Confirm from '@Components/Modal/Confirm';
 *
 * export default class {
 *   render() {
 *      return (
 *        <Confirm
 *          title="Title text..."
 *          text="Some content..."
 *          confirmLabel="Confirm"
 *          cancelLabel="Cancel"
 *          onConfirm={...}
 *          onCancel={...}
 *        />
 *      );
 *   }
 * }
 * ```
 */
@bemDecorator('tas-modal')
export default class Confirm extends React.Component {
  static displayName = 'Confirm';

  static propTypes = {
    title: PropTypes.string,
    text: PropTypes.string,
    confirmLabel: PropTypes.string,
    cancelLabel: PropTypes.string,
    onConfirm: PropTypes.func,
    onCancel: PropTypes.func,
    hideOnControl: PropTypes.bool,
    disabled: PropTypes.bool,
  };

  static defaultProps = {
    title: 'Confirm',
    text: '',
    confirmLabel: 'Confirm',
    cancelLabel: 'Cancel',
    hideOnControl: false,
    disabled: false,
    onConfirm() { },
    onCancel() { },
  };

  render(bem) {
    const {
      name, title, text, confirmLabel, cancelLabel,
      onConfirm, onCancel, disabled, hideOnControl
    } = this.props;

    return (
      <Modal
        name={name}
        ref={(ref) => { this.$modal = ref && ref.getWrappedInstance(); }}
        header={(
          <Title heading={4} theme="custom">{title}</Title>
        )}
        footer={(
          <div className={bem('confirm-buttons')}>
            <div className={bem('confirm')}>
              <Button
                theme="button-link"
                disabled={disabled}
                onClick={(e) => {
                  hideOnControl && this.$modal.hide();
                  onConfirm(e, this.$modal);
                }}
              >
                {confirmLabel}
              </Button>
            </div>

            <div className={bem('cancel')}>
              <Button
                theme="button-link"
                disabled={disabled}
                onClick={(e) => {
                  hideOnControl && this.$modal.hide();
                  onCancel(e, this.$modal);
                }}
              >
                {cancelLabel}
              </Button>
            </div>
          </div>
        )}
        onClose={onCancel}
      >
        <span className={bem('confirm-text')}>{text}</span>
      </Modal>
    );
  }
}
