import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';
import { connect } from 'react-redux';

import Icon from '../Icon';

import { showModal, hideModal, hideAllModal } from './redux';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import Modal from '@Components/Modal';
 *
 * export default class {
 *   render() {
 *      return (
 *        <Modal
 *          header="Header"
 *          footer="Maybe some buttons..."
 *          onClose={...}
 *        >
 *          Some content...
 *        </Modal>
 *      );
 *   }
 * }
 * ```
 */
@connect((state, { name }) => ({
  modal: (
    typeof state.get === 'function' ? state.get('modal')[name] : state.modal[name]
  ),
}), { hideModal }, null, { withRef: true })
@bemDecorator('tas-modal')
export default class Modal extends React.Component {
  static displayName = 'Modal';

  static propTypes = {
    name: PropTypes.string.isRequired,
    onClose: PropTypes.func,
    header: PropTypes.any,
    children: PropTypes.any,
    footer: PropTypes.any,
  };

  static defaultProps = {
    onClose() {
    },
    header: null,
    children: null,
    footer: null,
  };

  state = {
    hide: false,
  };

  componentWillUnmount() {
    this.props.hideModal(this.props.name);
  }

  handleClose(e) {
    this.setState({ hide: true });
    this.props.onClose(e);
  }

  hide() {
    this.setState({ hide: true });
  }

  render(bem) {
    const { hide } = this.state;

    const {
      header, children, footer, modal
    } = this.props;

    const meta = modal && modal.options || {};

    if (!modal) {
      return null;
    }

    return (
      <div
        id={`modal-${modal.name}`}
        className={bem({ hide })}
        onAnimationEnd={() => {
          hide && this.props.hideModal(this.props.name);
          this.setState({ hide: false });
        }}
      >
        <div
          className={bem('background')}
          onClick={e => this.handleClose(e)}
          role="presentation"
        />

        <section className={bem('dialog')}>
          <header className={bem('header')}>
            {header}

            <div
              className={bem('close')}
              onClick={e => this.handleClose(e)}
              role="presentation"
            >
              <Icon name="close" size={14} />
            </div>
          </header>

          <article className={bem('content')}>
            {React.cloneElement(children, { meta })}
          </article>

          <footer className={bem('footer')}>{footer}</footer>
        </section>
      </div>
    );
  }
}

export const modal = name => ComposedComponent => (
  connect(null, dispatch => ({
    show: dispatch(showModal(name)),
    hide: dispatch(hideModal(name)),
    hideAll: dispatch(hideAllModal()),
  }))(ComposedComponent)
);
