import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';

import Tabs, { Tab } from 'material-ui/Tabs';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import MultiTabs from '@Components/MultiTabs';
 *
 * export default class {
 *   render() {
 *     return (
 *       <MultiTabs
 *        tabs={[
 *          'first tab',
 *          'second tab',
 *        ]}
 *       >
 *        <FirstTab/>
 *        <SecondTab/>
 *       </MultiTabs>
 *     );
 *   }
 * }
 * ```
 */
@bemDecorator('multi-tabs')
export default class MultiTabs extends React.Component {
  static displayName = 'MultiTabs';

  static propTypes = {
    tabs: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.string, PropTypes.number])).isRequired,
    /**
     * Callback which will be fired when tabs change
     */
    onChange: PropTypes.func,
    /**
     * Index of active tab from tabs array
     */
    active: PropTypes.number,

    /**
     * Set default border to tabs
     */
    defaultBorder: PropTypes.bool,
    scrollable: PropTypes.bool,
    scrollButtons: PropTypes.oneOf(['on', 'off', 'auto', '']),
  };

  static defaultProps = {
    onChange: () => {
    },
    active: 0,
    defaultBorder: true,
    scrollable: false,
    scrollButtons: '',
  };

  state = {
    value: this.props.active,
  };

  componentWillReceiveProps(newProps) {
    if (this.props.active !== newProps.active) {
      this.setState({ value: newProps.active });
    }
  }

  handleChange(value) {
    this.setState({ value });
    this.props.onChange(value);
  }

  render(bem) {
    const { tabs, children, defaultBorder, scrollable, scrollButtons } = this.props;
    const { value } = this.state;

    return (
      <div className={bem()}>
        <Tabs
          value={value}
          classes={{
            root: bem('tabs-root', { line: defaultBorder }),
          }}
          onChange={(e, value) => this.handleChange(value)}
          scrollable={scrollable}
          scrollButtons={scrollButtons}
        >
          {tabs.map((tab, index) => (
            <Tab
              key={index}
              disabled={tab.disabled}
              label={tab.label || tab}
              classes={{
                root: bem('tab', { disabled: tab.disabled }),
                rootInheritSelected: bem('tab', { selected: true }),
              }}
            />
          ))}
        </Tabs>

        {
          React.Children.toArray(children)[value]
        }
      </div>
    );
  }
}
