import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';
import { connect } from 'react-redux';

import Alert from '../Alert';

import { hideNotification, deleteNotification } from './redux';

import './index.scss';

/**
 * ## Usage
 *
 * ### In store
 *
 * ```javascript
 * import notificationsReducer, { notificationsMiddleware } from '@Components/Notifications/redux';
 *
 * const store = createStore(
 *   combineReducers({ notifications: notificationsReducer }),
 *   applyMiddleware(notificationsMiddleware)
 * );
 * ```
 *
 * ### In root container:
 *
 * ```javascript
 * import Notifications from '@Components/Notifications';
 *
 * export default class {
 *   render() {
 *     return (
 *       <Notifications />
 *     );
 *   }
 * }
 * ```
 *
 * ### Throw `NotificationError`
 *
 * ```javascript
 * throw new NotificationError({
 *   title: 'Throw error',
 *   message: 'Throw error message'
 * });
 * ```
 *
 * ### Actions:
 *
 * - `showNotification({ type: enum(success, warning, error), title: string, message: string })` - Show new notification
 * - `hideNotification(index = 0): any` - Hide notification by index
 * - `hideAllNotifications(): any` - Hide all notifications
 * - `deleteNotification(index = 0): any` - Delete notification on store
 */
@connect(store => ({
  notifications: (typeof store.get === 'function') ? store.get('notifications') : store.notifications
}), { hideNotification, deleteNotification })
@bemDecorator('notifications')
export default class Notifications extends React.PureComponent {
  static displayName = 'Notifications';

  static propTypes = {
    top: PropTypes.number,
    right: PropTypes.number,
    timeout: PropTypes.number,
  };

  static defaultProps = {
    top: 24,
    right: 24,
    timeout: 5 * 1000
  };

  render(bem) {
    const { notifications, top, right, timeout } = this.props;

    return (
      <ul className={bem()} style={{ top, right }}>
        {notifications.map((item, index) => (
          <li style={{ zIndex: -index }} className={bem('item', { open: item.open })} key={item.title}>
            <Alert
              open={item.open}
              type={item.type}
              timeout={timeout}
              onClose={() => {
                this.props.hideNotification(index);
              }}
              onExit={() => {
                this.props.deleteNotification(index);
              }}
            >
              <div dangerouslySetInnerHTML={{ __html: item.message }} />
            </Alert>
          </li>
        ))}
      </ul>
    );
  }
}
