import React from 'react';
import PropTypes from 'prop-types';
import classNames from 'classnames';

import TextInput from '../TextInput';
import PasswordEye from './Eye';
import PasswordHints from './Hints';

import './index.scss';

/**
 * ## Usage
 * ```
 * <PasswordInput
 *  label="Password"
 *  getPasswordSafetyClass={value => value % 5}
 * />
 * ```
 * ## Other Properties
 * Any other properties supplied will be spread to the TextInput element
 */
export default class PasswordInput extends React.PureComponent {
  static displayName = 'PasswordInput';

  static propTypes = {
    /**
     * Enable Password redaction feature
     */
    eye: PropTypes.bool,
    /**
     * List of password tips texts, left it empty to disable feature
     */
    hints: PropTypes.arrayOf(PropTypes.string),
    /**
     * Class name of root div
     */
    rootClassName: PropTypes.string,
    /**
     * Function witch return safety mark of target password from 0 to 4,
     * left it empty to disable feature
     */
    getPasswordSafetyClass: PropTypes.func,

    input: PropTypes.object,
  };

  static defaultProps = {
    eye: true,
    hints: [],
    rootClassName: '',
    getPasswordSafetyClass: null,
    input: {
      onFocus: () => {},
      onBlur: () => {},
    }
  };

  state = {
    type: this.props.type, // props type is a real type, while state type is a current type
    // (it can be temporary changed for password demonstration)
    isInFocus: false,
  };

  componentWillReceiveProps({ type }) {
    this.setState({ type });
  }

  getPasswordSafetyClass(passwordValue) {
    const { getPasswordSafetyClass } = this.props;
    if (typeof getPasswordSafetyClass !== 'function') {
      return '';
    }
    const score = getPasswordSafetyClass(passwordValue);
    const scoreMap = ['yellow', 'yellow', 'green', 'green', 'strong-green'];
    return scoreMap[score];
  }

  changeInputType(type) {
    this.setState({ type });
  }

  render() {
    const {
      eye, hints, rootClassName, ...otherProps
    } = this.props;
    const { type = 'password', isInFocus } = this.state;
    return (
      <div className={classNames('password-input', rootClassName)}>
        <div className="password-input__wrapper">
          <TextInput
            {...{
              ...otherProps,
              innerProps: {
                className: classNames(
                  'password-input__input',
                  this.getPasswordSafetyClass(otherProps.input ? otherProps.input.value : '')
                )
              },
              input: {
                ...otherProps.input,
                type,
                onFocus: (e) => {
                  otherProps.input.onFocus(e.target.value);
                  this.setState({ isInFocus: true });
                },
                onBlur: (e) => {
                  otherProps.input.onBlur(e.target.value);
                  this.setState({ isInFocus: false });
                },
              },
            }}
          />
          {eye && <PasswordEye {...{ type }} onChange={type => this.changeInputType(type)} />}
          {hints && isInFocus && <PasswordHints {...{ hints }} />}
        </div>
      </div>
    );
  }
}
