import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';

import Input, { InputLabel } from 'material-ui/Input';
import { MenuItem } from 'material-ui/Menu';
import { FormControl, FormHelperText } from 'material-ui/Form';
import SelectUI from 'material-ui/Select';
import { LinearProgress } from 'material-ui/Progress';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import Example from '@Components/Select';
 *
 * export default class {
 *   render() {
 *     return (
 *       <Select />
 *     );
 *   }
 * }
 * ```
 */
@bemDecorator('select')
export default class Select extends React.PureComponent {
  static displayName = 'Select';

  static propTypes = {
    label: PropTypes.string,
    disabled: PropTypes.bool,
    input: PropTypes.object,
    innerProps: PropTypes.object,
    meta: PropTypes.object,
    id: PropTypes.string,
    helperText: PropTypes.string,
    list: PropTypes.array,
    emptyText: PropTypes.any,
    theme: PropTypes.oneOf([
      'default',
      'payments-form',
      'autofill-form',
    ]),
    loading: PropTypes.bool,
  };

  static defaultProps = {
    id: undefined,
    helperText: undefined,
    emptyText: undefined,
    disabled: false,
    label: 'Label',
    input: {
      value: '',
      name: '',
    },
    meta: {},
    innerProps: {},
    list: [],
    theme: 'default',
    loading: false,
  };

  componentDidUpdate(prevProps) {
    if (prevProps.meta.error &&
      (prevProps.meta.submitFailed === false && this.props.meta.submitFailed)) {
      const {
        input: { name }
      } = this.props;

      if (!document.getElementsByClassName('text-input__input_error').length) {
        const errorField = document
          .getElementsByClassName('select__input_error')[0];
        const inputFromDom = errorField
          .getElementsByTagName('input')[0];

        if (name === inputFromDom.name) {
          const focusedField = errorField.querySelectorAll('[tabindex="0"]')[0];

          focusedField && focusedField.focus();
        }
      }
    }
  }

  render(bem) {
    const {
      label, input, innerProps, meta, list,
      helperText, id, disabled, emptyText, loading,
    } = this.props;

    const error = (
      meta.touched || (meta.visited && !meta.active) || meta.submitFailed
    ) && meta.error;

    return (
      <FormControl
        disabled={disabled || loading}
        error={error}
        className={bem({ theme: this.props.theme, loading })}
      >
        <InputLabel
          htmlFor={id}
          FormControlClasses={{
            root: bem('label'),
            focused: bem('label', { focused: true }),
          }}
        >
          {label}
        </InputLabel>
        <SelectUI
          {...innerProps}
          {...input}
          input={<Input
            classes={{
              root: bem('input'),
              focused: bem('input', { focused: true }),
              error: bem('input', { error: true }),
            }}
            name={input.name}
            id={id}
          />}
          onBlur={null}
        >
          {emptyText && <MenuItem value=""><em>{emptyText}</em></MenuItem>}
          {list.map(({ value, label }) => (
            <MenuItem key={`${value}_${Date.now()}`} value={value}>{label}</MenuItem>
          ))}
        </SelectUI>

        {helperText && <FormHelperText className={bem('help-text')}>{helperText}</FormHelperText>}

        {error && <FormHelperText className={bem('help-text', { error: true })}>{error}</FormHelperText>}

        {loading && <LinearProgress classes={{ root: bem('progress') }} variant="query" />}
      </FormControl>
    );
  }
}
