import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';
import TableUI, { TableBody, TableCell, TableRow } from 'material-ui/Table';
import Checkbox from '../Checkbox';
import TableHead from './Head';
import TableFooter from './Footer';
import TableToolBar from './ToolBar';
import CircularProgress from '../CircularProgress';

import './index.scss';

let counter = 0;
const createData = (props) => {
  counter += 1;
  return {
    id: props.meta && props.meta.id ? props.meta.id : counter,
    visible: true,
    ...props,
  };
};

/**
 * ## Usage
 *
 * ```javascript
 * import Table from '@Components/Table';
 *
 * export default class {
 *   render() {
 *     return (
 *       <Table
 *          columnData={[
 *            {id: "col1", label:"Label1"},
 *            {id: "col2", label:"Label1"},
 *            {id: "col3", label:"Label1"},
 *          ]}
 *          rowData={[
 *            {col1: {value: "val1", rowSpan: 2}, col2: "val2", col3: "val3"},
 *            {col2: "val1", col3: "val3"},
 *          ]}
 *          labelNoSearchResult="Have no result"
 *       />
 *     );
 *   }
 * }
 * ```
 */
@bemDecorator('tas-table')
export default class Table extends React.Component {
  static displayName = 'Table';

  static propTypes = {
    /** An array of data from which head of table constructed */
    columnData: PropTypes.arrayOf(PropTypes.shape({
      /** name of property from rowData for column constructing */
      id: PropTypes.string.isRequired,
      /** name of property from rowData for column sorting */
      orderBy: PropTypes.string,
      /** align text at column to right side, like 'numeric' style */
      alignRight: PropTypes.bool,
      /** visible column name */
      label: PropTypes.any.isRequired,
      /** width of column */
      width: PropTypes.string,
    })).isRequired,
    rowData: PropTypes.array.isRequired,
    /** the array of properties that will be searched */
    keysForSearch: PropTypes.array,
    /** main table label */
    mainLabel: PropTypes.any,
    /** callback triggered for change page, return current page and next page */
    onChangePage: PropTypes.func,
    /** callback triggered for change count of rows per page, return current count and next count */
    onChangeRowsPerPage: PropTypes.func,
    /** callback triggered for column sort, return sorted data, column name and order */
    onChangeSort: PropTypes.func,
    /** callback triggered for click on the row, return row data */
    onRowClick: PropTypes.func,
    /** callback triggered for all rows select, return 'true' or 'false' */
    onSelectAll: PropTypes.func,
    /** callback triggered for one row select, return data of all selected rows */
    onSelectOne: PropTypes.func,
    /** count of rows per page, for pagination */
    rowsPerPage: PropTypes.number,
    /** count of rows per page for user select, for pagination */
    rowsPerPageOptions: PropTypes.array,
    /** position at string from which search start, for search */
    searchStartFrom: PropTypes.number,
    theme: PropTypes.oneOf([
      'default',
      'payments-table',
    ]),
    /** enabling pagination footer, for pagination */
    withPagination: PropTypes.bool,
    /** enabling checkboxes */
    withCheckboxes: PropTypes.bool,
    /** enabling hover effect on row */
    withHover: PropTypes.bool,
    /** enabling sortable */
    withSort: PropTypes.bool,
    /** enabling search bar */
    withSearchBar: PropTypes.bool,
    /** label "Транзакцій на сторінку" */
    labelRowsPerPage: PropTypes.string,
    /** label "XX транзакцій обрано" */
    labelChosen: PropTypes.string,
    /** label "Збігів не знайдено" */
    labelNoSearchResult: PropTypes.string,
    /** set table-layout fixed  */
    fixed: PropTypes.bool,
    isPreload: PropTypes.bool,
    toolsBar: PropTypes.any,
    order: PropTypes.oneOf([
      'asc',
      'desc',
    ]),
    count: PropTypes.number,
    currentPage: PropTypes.number,
    defaultOrderBy: PropTypes.string,
    sticky: PropTypes.oneOfType([PropTypes.bool, PropTypes.number]),
  };

  static defaultProps = {
    onRowClick: () => {
    },
    onSelectAll: () => {
    },
    onSelectOne: () => {
    },
    onChangePage: () => {
    },
    onChangeRowsPerPage: () => {
    },
    onChangeSort: () => {
    },
    rowsPerPage: 5,
    rowsPerPageOptions: [5, 10, 25],
    searchStartFrom: 3,
    theme: 'default',
    withPagination: false,
    withCheckboxes: false,
    withHover: false,
    withSort: false,
    withSearchBar: false,
    keysForSearch: [],
    mainLabel: '',
    labelRowsPerPage: '',
    labelChosen: '',
    labelNoSearchResult: '',
    fixed: false,
    isPreload: false,
    toolsBar: '',
    order: 'asc',
    defaultOrderBy: '',
    count: undefined,
    currentPage: undefined,
    sticky: false,
  };

  state = {
    order: this.props.order || 'asc',
    orderBy: this.defaultOrderBy,
    selected: [],
    data: this.props.rowData.map(createData),
    page: 0,
    rowsPerPage: this.props.rowsPerPage,
  };

  searchQuery = null;

  componentWillMount() {
    const { defaultOrderBy } = this.props;
    const { orderBy } = this.state;

    if (defaultOrderBy) {
      this.handleRequestSort({ column: { id: orderBy, orderBy } });
    }
  }

  componentDidMount() {
    const { sticky } = this.props;

    if (sticky) {
      function handleScroll() {
        const layoutTop = Number.isInteger(sticky) ? sticky : 0;
        const { top: tableTop, width: tableWidth } = this.$tableComponent.getBoundingClientRect();

        if (tableTop <= (layoutTop + 10)) {
          this.$tableToolbar.style.width = `${tableWidth}px`;

          if (Number.isInteger(sticky)) {
            this.$tableToolbar.style.top = `${sticky}px`;
            this.$tableComponent.querySelector('thead').style.top = `${sticky + 65}px`;
          }

          Array.from(this.$tableComponent.querySelectorAll('thead th'))
            .forEach((cell) => {
              const { width: cellWidth } = cell.getBoundingClientRect();
              cell.style.width = `${cellWidth}px`;
            });

          // this.$tableHiddenHeadRow.style.height = `${this.$tableComponent.querySelector('thead').getBoundingClientRect().height}px`;
          this.$tableComponent.classList.add('tas-table_sticky');
        } else {
          // this.$tableHiddenHeadRow.style.height = '0px';
          this.$tableComponent.classList.remove('tas-table_sticky');
        }
      }

      window.onscroll = handleScroll.bind(this);
    }
  }

  componentWillReceiveProps({ rowData }) {
    const { orderBy = 'details', order } = this.state;
    const newData = rowData.map(createData);

    const data =
      !this.props.withSort
        ? newData
        : (order === 'desc')
          ? newData.sort((a, b) => (b[orderBy] < a[orderBy] ? -1 : 1))
          : newData.sort((a, b) => (a[orderBy] < b[orderBy] ? -1 : 1));

    const total = data.map(({ id }) => id);
    const selected = this.state.selected.filter(id => total.includes(id));

    this.setState({ data, selected }, () => {
      if (this.searchQuery) {
        this.handleChangeSearchValue(this.searchQuery);
      }
    });
  }

  componentWillUnmount() {
    window.onscroll = null;
  }

  get defaultOrderBy() {
    const { defaultOrderBy, columnData, withSort } = this.props;

    if (withSort) {
      return defaultOrderBy || columnData.find(item => item.orderBy).orderBy;
    }

    return '';
  }

  get dataWithPagination() {
    const { data, rowsPerPage, page } = this.state;

    if (this.props.withPagination) {
      return data
        .filter(item => item.visible)
        .slice(page * rowsPerPage, (page * rowsPerPage) + rowsPerPage);
    }

    return data.filter(item => item.visible);
  }

  handleRequestSort({ column: { id, orderBy } }) {
    let order = 'desc';

    if (this.state.orderBy === id && this.state.order === 'desc') {
      order = 'asc';
    }

    const data =
      order === 'desc'
        ? this.state.data.sort((a, b) => (b[orderBy] < a[orderBy] ? -1 : 1))
        : this.state.data.sort((a, b) => (a[orderBy] < b[orderBy] ? -1 : 1));

    this.setState({ data, order, orderBy: id });
    this.props.onChangeSort({ data, order, orderBy: id });
  }

  handleSelectAllClick = async (selectAll) => {
    const selected = selectAll ?
      this.state.data.filter(n => n.visible).map(n => n.id)
      : [];

    await this.setState({ selected });

    this.props.onSelectAll({ selectAll, selected: this.state.data });
  };

  handleCheckboxChange = async (id) => {
    const { selected } = this.state;
    const selectedIndex = selected.indexOf(id);
    let newSelected = [];

    if (selectedIndex === -1) {
      newSelected = newSelected.concat(selected, id);
    } else if (selectedIndex === 0) {
      newSelected = newSelected.concat(selected.slice(1));
    } else if (selectedIndex === selected.length - 1) {
      newSelected = newSelected.concat(selected.slice(0, -1));
    } else if (selectedIndex > 0) {
      newSelected = newSelected.concat(
        selected.slice(0, selectedIndex),
        selected.slice(selectedIndex + 1),
      );
    }

    const newData = newSelected.reduce((target, item) => [...target, this.state.data.find(({ id }) => id === item)], []);

    await this.setState({ selected: newSelected });
    await this.props.onSelectOne({ selected: newData });
  };

  handleChangePage(page) {
    this.props.onChangePage({
      current: this.state.page,
      next: page,
    });
    this.setState({ page });
  }

  handleChangeRowsPerPage({ target: { value } }) {
    this.props.onChangeRowsPerPage({
      current: this.state.rowsPerPage,
      next: value,
    });
    this.setState({ rowsPerPage: value });
  }

  handleChangeSearchValue(value) {
    let newData = [];
    if (value.length >= this.props.searchStartFrom) {
      newData = this.state.data.map(item => ({
        ...item,
        visible: this.props.keysForSearch.some((key) => {
          if (key in item) {
            return item[key]
              .toString()
              .toLowerCase()
              .indexOf(value.toLowerCase()) !== -1;
          }
          return true;
        }),
      }));
    } else {
      newData = this.state.data.map(item => ({ ...item, visible: true }));
    }

    this.searchQuery = value;

    this.setState({ data: newData });
  }

  render(bem) {
    const {
      columnData, withPagination, rowsPerPageOptions, withCheckboxes, withSort, withHover,
      mainLabel, labelNoSearchResult, labelChosen, labelRowsPerPage, withSearchBar, fixed,
      isPreload, toolsBar, count, currentPage,
    } = this.props;
    const {
      data, order, orderBy, selected, rowsPerPage, page
    } = this.state;
    const visibleRowsCount = data.filter(item => item.visible).length;
    const emptyRowsCount = rowsPerPage - Math.min(rowsPerPage, visibleRowsCount - (page * rowsPerPage)); // eslint-disable-line max-len
    const colSpan = withCheckboxes ? columnData.length + 1 : columnData.length;
    const haveVisibleRows = data.some(item => item.visible);

    return (
      <section
        ref={(a) => {
          this.$tableComponent = a;
        }}
        className={bem({ theme: this.props.theme })}
      >
        <TableToolBar
          innerRef={(a) => {
            this.$tableToolbar = a;
          }}
          labelChosen={labelChosen}
          onChangeSearchValue={value => this.handleChangeSearchValue(value)}
          mainLabel={mainLabel}
          selectedCount={selected.length}
          withSearchBar={withSearchBar}
          tools={toolsBar}
        />

        <TableUI className={bem('table', { fixed })}>
          <TableHead
            numSelected={selected.length}
            order={order}
            orderBy={orderBy}
            onSelectAllClick={(event, selectAll) => this.handleSelectAllClick(selectAll)}
            onRequestSort={data => this.handleRequestSort(data)}
            rowCount={data.length}
            columnData={columnData}
            withCheckboxes={withCheckboxes}
            withSort={withSort}
          />

          <thead style={{ visibility: 'hidden' }}>
            <tr
              className={bem('row', { hidden: true })}
              // ref={(a) => {
              //   this.$tableHiddenHeadRow = a;
              // }}
            >
              {withCheckboxes && (<th />)}
              {columnData.map(column => (<th key={column.id} />))}
            </tr>
          </thead>

          {!isPreload ? (
            <TableBody>
              {haveVisibleRows ? (
                this.dataWithPagination.map(item => (
                  <TableRow
                    hover={withHover}
                    key={item.id}
                    className={bem('row', { body: true, 'no-hover': !this.props.withHover })}
                  >
                    {withCheckboxes && (
                      <TableCell
                        padding="none"
                        className={bem('cell', { checkbox: true })}
                      >
                        <Checkbox
                          input={{
                            checked: this.state.selected.indexOf(item.id) !== -1,
                            onChange: () => this.handleCheckboxChange(item.id),
                          }}
                        />
                      </TableCell>
                    )}

                    {columnData
                      .map(i => i.id)
                      .filter(key => item[key])
                      .map((key, index) => (
                        <TableCell
                          key={key}
                          className={bem('cell', { first: index === 0, last: index === (columnData.length - 1) })}
                          numeric={columnData.find(i => i.id === key).alignRight}
                          padding="none"
                          rowSpan={item[key].rowSpan || 1}
                          onClick={() => this.props.onRowClick(item)}
                        >
                          {(item[key].rowSpan || item[key].colSpan) ? item[key].value : item[key]}
                        </TableCell>
                      ))}
                  </TableRow>
                ))
              ) : (
                <TableRow style={withPagination ? { height: 49 * emptyRowsCount } : {}}>
                  <TableCell colSpan={colSpan} style={{ verticalAlign: 'middle' }}>
                    <section className={bem('no-search-result')}>
                      <div className={bem('no-search-icon')} />
                      {labelNoSearchResult}
                    </section>
                  </TableCell>
                </TableRow>
              )}

              {withPagination && data.length > 0 && emptyRowsCount > 0 && haveVisibleRows && (
                <TableRow style={{ height: 49 * emptyRowsCount }}>
                  <TableCell colSpan={colSpan} />
                </TableRow>
              )}
            </TableBody>
          ) : (
            <TableBody>
              <TableRow style={withPagination ? { height: 49 * rowsPerPage } : {}}>
                <TableCell colSpan={colSpan} style={{ verticalAlign: 'middle' }}>
                  <CircularProgress isPreload />
                </TableCell>
              </TableRow>
            </TableBody>
          )}

          {withPagination && data.length > 0 && (
            <TableFooter
              colSpan={colSpan}
              count={count || visibleRowsCount}
              labelRowsPerPage={labelRowsPerPage}
              rowsPerPage={rowsPerPage}
              rowsPerPageOptions={rowsPerPageOptions}
              page={currentPage || page}
              onChangePage={(event, page) => this.handleChangePage(page)}
              onChangeRowsPerPage={event => this.handleChangeRowsPerPage(event)}
            />
          )}
        </TableUI>
      </section>
    );
  }
}
