import React from 'react';
import PropTypes from 'prop-types';
import bemDecorator from 'cn-decorator';

import Input, { InputLabel } from 'material-ui/Input';
import { FormControl, FormHelperText } from 'material-ui/Form';
import { LinearProgress } from 'material-ui/Progress';
import Icon from '../Icon';

import './index.scss';

/**
 * ## Usage
 *
 * ```javascript
 * import { Field } from 'redux-form/immutable';
 * import TextInput from '@Components/TextInput';
 *
 * export default class {
 *   render() {
 *     return (
 *       <Field
 *          name="myInput"
 *          component={TextInput}
 *          label="Label text"
 *       />
 *     );
 *   }
 * }
 * ```
 */
@bemDecorator('text-input')
export default class TextInput extends React.Component {
  static displayName = 'TextInput';

  static propTypes = {
    label: PropTypes.string,
    disabled: PropTypes.bool,
    prefix: PropTypes.string,
    suffix: PropTypes.string,
    input: PropTypes.object,
    autoFocus: PropTypes.bool,
    innerProps: PropTypes.object,
    meta: PropTypes.object,
    id: PropTypes.string,
    helperText: PropTypes.string,
    multiline: PropTypes.bool,
    rows: PropTypes.number,
    rowsMax: PropTypes.number,
    inputComponent: PropTypes.func,
    randomName: PropTypes.bool,
    inputProps: PropTypes.object,
    theme: PropTypes.oneOf([
      'default',
      'payments-form',
      'payments-form-numbers',
      'autofill-form',
    ]),
    loading: PropTypes.bool,
  };

  static defaultProps = {
    autoFocus: false,
    disabled: false,
    label: undefined,
    prefix: undefined,
    suffix: undefined,
    helperText: undefined,
    id: undefined,
    inputComponent: undefined,
    innerProps: {},
    inputProps: {},
    input: {},
    meta: {},
    multiline: false,
    rows: 1,
    rowsMax: 1,
    theme: 'default',
    randomName: false,
    loading: false,
  };

  componentDidUpdate(prevProps) {
    if (prevProps.meta.error &&
      (prevProps.meta.submitFailed === false && this.props.meta.submitFailed)) {
      const {
        inputProps,
        input
      } = this.props;

      const name = this.inputName || input.name || inputProps.name;

      const inputFromDom = document
        .getElementsByClassName('text-input__input_error')[0]
        .getElementsByTagName('input')[0];

      if (name === inputFromDom.name) {
        inputFromDom && inputFromDom.focus();
      }
    }
  }

  inputName = (
    this.props.randomName ? Math.random().toString(32).substr(2, 10) : undefined
  );

  render(bem) {
    const {
      label, input, autoFocus, innerProps, prefix,
      helperText, id, disabled, suffix, meta, multiline, rows,
      rowsMax, inputComponent, inputProps, loading
    } = this.props;

    const error = (
      meta.touched
      || (meta.visited && !meta.active)
      || meta.submitFailed
      || (!!meta.initial && !!input.value)
    ) && meta.error;

    return (
      <section className={bem({ theme: this.props.theme, loading, })}>
        {prefix && <div className={bem('prefix')}><span>{prefix}</span></div>}

        <FormControl error={error} className={bem('control', { error })}>
          <InputLabel
            htmlFor={id}
            error={error}
            FormControlClasses={{
              root: bem('label'),
              focused: bem('label', { focused: true }),
              error: bem('label', { error: true }),
            }}
            classes={{
              shrink: bem('label', { shrink: true }),
            }}
          >{label}
          </InputLabel>
          <Input
            multiline={multiline}
            rows={rows}
            rowsMax={rowsMax}
            autoFocus={autoFocus}
            error={error || undefined}
            disabled={disabled || loading}
            id={id}
            inputComponent={inputComponent}
            inputProps={{
              ...inputProps,
              name: this.inputName || input.name || inputProps.name,
            }}
            classes={{
              root: bem('input'),
              focused: bem('input', { focused: true }),
              error: bem('input', { error: true }),
            }}
            {...innerProps}
            {...input}
          />
          <span className={bem('icon')}>
            <Icon name="edit" size="14" />
          </span>
        </FormControl>
        {suffix && <div className={bem('suffix')}><span>{suffix}</span></div>}

        {helperText && <FormHelperText className={bem('help-text')}>{helperText}</FormHelperText>}

        {error && <FormHelperText className={bem('help-text', { error: true })}>{error}</FormHelperText>}

        {loading && <LinearProgress classes={{ root: bem('progress') }} variant="query" />}
      </section>
    );
  }
}

