import React from 'react';
import PropTypes from 'prop-types';
import Typography from 'material-ui/Typography';
import classNames from 'classnames';

import './index.scss';


/**
 * ## About Component
 * Title element supports classic html heading system, like h1-h6, with
 * paragraph property, as modern display property, with display property.
 * ## Usage
 *
 * ```javascript
 * import Title from '@Components/Title';
 *
 * export default class {
 *   render() {
 *     return (
 *       <Title heading={2}>Hola!</Title> {# is equivalent to <h2> #}
 *     );
 *   }
 * }
 * ```
 * ## Other Properties
 * Any other properties supplied will be spread to the *material-ui* [Typography component](https://material-ui-next.com/api/typography/)
 */
export default class Title extends React.PureComponent {
  static displayName = 'Title';

  static propTypes = {
    /**
     * Corresponds to native html headings, like <h1>, <h2> ... <h6>
     */
    heading: PropTypes.oneOf([1, 2, 3, 4, 5, 6]),
    /**
     * Corresponds to bootstrap's modern css headings
     */
    display: PropTypes.oneOf([1, 2, 3, 4]),
    theme: PropTypes.oneOf(['default', 'custom']),
  };

  static defaultProps = {
    heading: undefined,
    display: undefined,
    theme: 'default',
  };

  render() {
    const {
      theme,
      heading, display, className, ...otherProps
    } = this.props;
    return (
      <Typography
        className={classNames('MuiTypography-heading', `MuiTypography-heading__theme_${theme}`, {
          [className]: className,
        [`MuiTypography-h${heading}`]: heading
        })}
        type={display ?
          `display${display}` : heading && 'headline'
        }
        component={heading ? `h${heading}` : undefined}
        {...otherProps}
      />
    );
  }
}
